/* This file is part of rpipe
   Copyright (C) 2018 Sergey Poznyakoff
   License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>
   This is free software: you are free to change and redistribute it.
   There is NO WARRANTY, to the extent permitted by law.
*/
#include <sys/types.h>
#include <sys/socket.h>
#include <string.h>
#include <signal.h>
#include <netdb.h>
#include <stdio.h>
#include <errno.h>
#include <sysexits.h>
#include "rpipe.h"

void
client_sig_handler(int sig)
{
	error(EX_OSERR, 0, "got signal %s", strsignal(sig));
}
		
static int client_sigv[] = {
	SIGCHLD,
	SIGTERM,
	SIGQUIT,
	SIGINT,
	SIGHUP,
	SIGUSR1,
	SIGALRM,
	SIGPIPE
};
static int client_sigc = sizeof(client_sigv)/sizeof(client_sigv[0]);

static void
trim(char *buf)
{
	size_t len = strlen(buf);
	if (len > 0 && buf[len-1] == '\n') {
		--len;
		if (len > 0 && buf[len-1] == '\r')
			--len;
		buf[len] = 0;
	}
}

void
rpipe_client(struct addrinfo *ap, int argc, char **argv)
{
	int fd;
	FILE *fp;
	int i, c;
	char buf[IO_BUF_SIZE];
	
	signal_setup(client_sig_handler, client_sigv, client_sigc);
	
	fd = socket(ap->ai_addr->sa_family, SOCK_STREAM, 0);
	if (fd == -1)
		error(EX_OSERR, errno, "can't create socket");

	if (connect(fd, ap->ai_addr, ap->ai_addrlen)) 
		error(EX_OSERR, errno, "connect");

	fp = fdopen(fd, "w+");

	for (i = 0; i < argc; i++) {
		if (i > 0)
			fputc(' ', fp);
		fprintf(fp, "%s", argv[i]);
	}
	fputc('\r', fp);
	fputc('\n', fp);
	if (ferror(fp))
		error(EX_OSERR, 0, "write error");

	if (!fgets(buf, sizeof(buf), fp))
		error(EX_OSERR, errno, "error reading response");
	trim(buf);
	if (buf[0] == '-')
		error(EX_UNAVAILABLE, 0, "%s", buf + 2);
	else if (buf[0] != '+')
		error(EX_UNAVAILABLE, 0, "unexpected response: %s", buf);
	
	while ((c = getc(stdin)) != EOF) {
		putc(c, fp);
	}
	if (ferror(fp))
		error(EX_OSERR, 0, "write error");
	fflush(fp);
	shutdown(fd, SHUT_WR);

	if (!fgets(buf, sizeof(buf), fp))
		error(EX_OSERR, errno, "error reading response");
	fclose(fp);

	if (buf[0] == '-')
		error(EX_UNAVAILABLE, 0, "%s", buf + 2);
	else if (buf[0] != '+')
		error(EX_UNAVAILABLE, 0, "unexpected response: %s", buf);
	exit(atoi(buf+2));
}
